//********************************************************************
//*** ResourcePool/BatchCommand.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: BatchCommand.cc,v 1.8 2008/01/27 16:42:27 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_BATCHCOMMAND_HH
#include "BatchCommand.hh"
#endif

namespace fatalmind {

template<class CommandType>
const int BatchCommand<CommandType>::NOT_STARTED = -1;

template<class CommandType>
const int BatchCommand<CommandType>::NO_FURTHER_ERROR = -2;

template<class CommandType>
BatchCommand<CommandType>::BatchCommand(bool a_cont) 
: CommandType()
, list()
, cont(a_cont)
, started(false)
, exec_pos(list.begin())
, errors()
, error_pos(0)
{
}

template<class CommandType>
BatchCommand<CommandType>::BatchCommand(const BatchCommand& rho) 
: CommandType(rho)
, list(rho.list)
, cont(rho.cont)
, started(rho.started)
, exec_pos(rho.exec_pos)
, errors(rho.errors)
, error_pos(rho.error_pos)
{
}

template<class CommandType>
void BatchCommand<CommandType>::add(const CommandType& cmd) 
{
	list.push_back(gc_ptr<CommandType>(cmd.template clone<CommandType>()));
}

template<class CommandType>
void BatchCommand<CommandType>::execute(typename CommandType::pool_type::resource_t& res) 
{
	const_iterator endi  = list.end();
	if ((! cont) || (! started)) {
		// re-start executing (in case !cont)
		// or initialize in case it's the first execution at all
		// (! started)
		exec_pos = list.begin();

		// begin of an emtpy vector is not a valid iterator :(
		errors.clear();
		errors.reserve(list.size()+2); // makes relocation impossible
		error_pos = 0;
	} 
	if (cont && started && (exec_pos != endi)) {
		// skipp the entry which caused an error 
		// if it is not yet at the end
		++exec_pos;
	}
	started=true;
	try {
		for (; exec_pos != endi; ++exec_pos) {
			(*exec_pos)->execute(res);
		}
	} catch (...) {
		errors.push_back(exec_pos - list.begin());
		throw; // re-throw
	}
	exec_pos = endi; // no error hppend
}

template<class CommandType>
int BatchCommand<CommandType>::getError()
{
	if (started) {
		if (error_pos < errors.size()) {
			return errors[error_pos++];
		} else {
			return NO_FURTHER_ERROR;
		}	
	} else {
		return NOT_STARTED;
	}
}
template<class CommandType>
typename BatchCommand<CommandType>::const_iterator BatchCommand<CommandType>::getExecPosition() const {
	return exec_pos;
}

template<class CommandType>
typename BatchCommand<CommandType>::const_iterator BatchCommand<CommandType>::end() const
{
	return list.end();
}

template<class CommandType>
typename BatchCommand<CommandType>::const_iterator BatchCommand<CommandType>::begin() const
{
	return list.begin();
}

template<class CommandType>
bool BatchCommand<CommandType>::empty() const
{
	return list.empty();
}

} // namespace fatalmind
